﻿using FlowerShopBusinessLogic.OfficePackage.HelperEnums;
using FlowerShopBusinessLogic.OfficePackage.HelperModels;

namespace FlowerShopBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToWord
    {
        public void CreateDoc(WordInfo info)
        {
            CreateWord(info);
            CreateParagraph(new WordParagraph
            {
                Texts = new List<(string, WordTextProperties)> { 
                    (info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    JustificationType = WordJustificationType.Center
                }
            });
            foreach (var flower in info.Flowers)
            {
                CreateParagraph(new WordParagraph
                {
                    Texts = new List<(string, WordTextProperties)> {
                        (flower.FlowerName, new WordTextProperties { Bold = true, Size = "24", }),
                        (flower.Price.ToString(), new WordTextProperties { Size = "24" })
                    },
                    TextProperties = new WordTextProperties
                    {
                        Size = "24",
                        JustificationType = WordJustificationType.Both
                    }

                });
            }
            SaveWord(info);
        }
		public void CreateShopsTable(WordInfo info)
		{
			CreateWord(info);

			CreateParagraph(new WordParagraph
			{
				Texts = new List<(string, WordTextProperties)> { (info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
				TextProperties = new WordTextProperties
				{
					Size = "24",
					JustificationType = WordJustificationType.Center
				}
			});

			CreateTable(new List<string> { "3000", "3000", "3000" });

			CreateRow(new WordParagraph
			{
				Texts = new List<(string, WordTextProperties)> { ("Название магазина", new WordTextProperties { Size = "24" }),
					("Адрес", new WordTextProperties { Size = "24" }), ("Дата открытия", new WordTextProperties { Size = "24" }) },
				TextProperties = new WordTextProperties
				{
					Size = "24",
					Bold = true,
					JustificationType = WordJustificationType.Center
				}
			});

			foreach (var shop in info.Shops)
			{
				CreateRow(new WordParagraph
				{
					Texts = new List<(string, WordTextProperties)> { (shop.ShopName, new WordTextProperties { Size = "22" }),
						(shop.Address, new WordTextProperties { Size = "22" }), (shop.DateOpening.ToString(), new WordTextProperties { Size = "22" })},
					TextProperties = new WordTextProperties
					{
						Size = "22",
						JustificationType = WordJustificationType.Center
					}
				});
			}

			SaveWord(info);
		}
		protected abstract void CreateWord(WordInfo info);
        protected abstract void CreateParagraph(WordParagraph paragraph);
		protected abstract void CreateTable(List<string> columns);
		protected abstract void CreateRow(WordParagraph paragraph);
		protected abstract void SaveWord(WordInfo info);
    }

}
